==> react-run-0.1.0.crate: react-run-0.1.0/src/lib.rs <==
pub mod config;
use clap::StructOpt;
use colored::Colorize;
use config::Args;
use open::that;
use std::{fs, io::prelude::*, net, process};

static TEMPLATE: &str = include_str!("template.html");

pub fn run() {
    let args = Args::parse();
    let app = react_app(&args.file);
    let port = format!("localhost:{}", args.port);

    let listener = match net::TcpListener::bind(&port) {
        Ok(listener) => listener,
        Err(e) => {
            eprintln!(
                "Cannot bind to port {}, probably is busy by other process: {}",
                &port, e
            );
            process::exit(1);
        }
    };

    println!("{}{}", "Listening on http://".blue(), &port.blue());

    that(format!("http://{}", &port)).unwrap();

    for stream in listener.incoming() {
        let stream = stream.unwrap();
        handle_connection(stream, &app);
    }
}

fn react_app(file: &str) -> String {
    let app = match fs::read_to_string(file) {
        Ok(app) => app,
        Err(e) => {
            eprintln!("Could not read file \"{}\": {}.", file.green(), e);
            process::exit(1);
        }
    };

    TEMPLATE.replace("// APP", &app)
}

fn handle_connection(mut stream: net::TcpStream, app: &str) {
    match stream.read(&mut [0; 1024]) {
        Ok(_) => println!("{}", "Request received.".green()),
        Err(error) => {
            eprintln!("Error reading the stream: {}", error);
            process::exit(1);
        }
    };

    let response = format!(
        "HTTP/1.1 200 OK\r\nContent-Length: {}\r\n\r\n{}",
        app.len(),
        app
    );

    match stream.write(response.as_bytes()) {
        Ok(_) => println!("[{}]", "Ping!".green()),
        Err(error) => {
            eprintln!("Could not write to stream: {}.", error);
            process::exit(1);
        }
    };

    stream.flush().unwrap();
}

==> react-run-0.2.0.crate: react-run-0.2.0/src/lib.rs <==
pub mod config;
use clap::StructOpt;
use colored::Colorize;
use config::Args;
use minreq::get;
use open::that;
use std::{fs, io::prelude::*, net, process};
use url::Url;

static TEMPLATE: &str = include_str!("template.html");

pub fn run() {
    let args = Args::parse();
    let app = react_app(&args.file);
    let port = format!("localhost:{}", args.port);

    let listener = match net::TcpListener::bind(&port) {
        Ok(listener) => listener,
        Err(e) => {
            eprintln!(
                "Cannot bind to port {}, probably is busy by other process: {}",
                &port, e
            );
            process::exit(1);
        }
    };

    println!("{}{}", "Listening on http://".blue(), &port.blue());

    if !args.simple {
        that(format!("http://{}", &port)).unwrap();
    };

    for stream in listener.incoming() {
        let stream = stream.unwrap();
        handle_connection(stream, &app);
    }
}

fn react_app(file: &str) -> String {
    let is_url = Url::parse(&file).is_ok();

    let app = if is_url {
        let resp = match get(&*file).send() {
            Ok(resp) => resp,
            Err(e) => {
                eprintln!("Error fetching that URL: {}", e);
                process::exit(1);
            }
        };

        let app = match resp.as_str() {
            Ok(app) => app,
            Err(e) => {
                eprintln!("Error parsing response as string: {}", e);
                process::exit(1);
            }
        };

        String::from(app)
    } else {
        match fs::read_to_string(file) {
            Ok(app) => app,
            Err(e) => {
                eprintln!("Could not read file \"{}\": {}.", file.green(), e);
                process::exit(1);
            }
        }
    };

    TEMPLATE.replace("// APP", &app)
}

fn handle_connection(mut stream: net::TcpStream, app: &str) {
    match stream.read(&mut [0; 1024]) {
        Ok(_) => println!("{}", "Request received.".green()),
        Err(error) => {
            eprintln!("Error reading the stream: {}", error);
            process::exit(1);
        }
    };

    let response = format!(
        "HTTP/1.1 200 OK\r\nContent-Length: {}\r\n\r\n{}",
        app.len(),
        app
    );

    match stream.write(response.as_bytes()) {
        Ok(_) => println!("[{}]", "Ping!".green()),
        Err(error) => {
            eprintln!("Could not write to stream: {}.", error);
            process::exit(1);
        }
    };

    stream.flush().unwrap();
}
